
#ifndef mac_802_15_4_timers_h
#define mac_802_15_4_timers_h

#include "mac.h"

class Mac802_15_4;

class Mac15Timer : public Handler {
public:
        Mac15Timer(Mac802_15_4* m, double s = 0) : mac(m), slottime(s) {
                busy_ = paused_ = 0; stime = rtime = 0.0;
        }

        virtual void handle(Event *e) = 0;

        virtual void start(double time);
        virtual void stop(void);
        virtual void pause(void) { assert(0); }
        virtual void resume(void) { assert(0); }

        inline int busy(void) { return busy_; }
        inline int paused(void) { return paused_; }
        inline double expire(void) {
                return ((stime + rtime) - Scheduler::instance().clock());
        }

protected:
        Mac802_15_4       *mac;
        int             busy_;
        int             paused_;
        Event           intr;
        double          stime;  // start time
        double          rtime;  // remaining time
        double          slottime;
};
 
/*
class BackoffTimer : public Mac15Timer {
public:
        BackoffTimer(Mac802_15_4 *m, double s) : Mac15Timer(m, s), difs_wait(0.0) {}

        void    start(int cw, int idle);
        void    handle(Event *e);
        void    pause(void);
        void    resume(double difs);
private:
        double  difs_wait;
};
*/

class DeferTimer : public Mac15Timer {
public:
        DeferTimer(Mac802_15_4 *m, double s) : Mac15Timer(m,s) {}

        void    start(double);
        void    handle(Event *e);
};

class IFTimer : public Mac15Timer {
public:
        IFTimer(Mac802_15_4 *m) : Mac15Timer(m) {}

        void    handle(Event *e);
};

/*
class NavTimer : public Mac15Timer {
public:
        NavTimer(Mac802_15_4 *m) : Mac15Timer(m) {}

        void    handle(Event *e);
};
*/

class RxTimer : public Mac15Timer {
public:
        RxTimer(Mac802_15_4 *m) : Mac15Timer(m) {}

        void    handle(Event *e);
};

class TxTimer : public Mac15Timer {
public:
        TxTimer(Mac802_15_4 *m) : Mac15Timer(m) {}

        void    handle(Event *e);
};

class BeaconTimer : public TimerHandler {
public:
	BeaconTimer(Mac802_15_4 *m) : TimerHandler() { mac_ = m; }
	void expire(Event *e); 
protected:
	Mac802_15_4 *mac_;
};

#endif /* __mac_15_4_timers_h__ */

