/* IEEE 802.15.4 Standard  */

/* Gang Lu
   Department of Electrical Engineering
   University of Southern California
*/
#ifndef MAC_802_15_4__TYPE_H
#define MAC_802_15_4_TYPE_H

#define HDR_MAC802_15_4(p) ((hdr_mac802_15_4 *)hdr_mac::access(p))
#define u_int64_t 			u_int32_t

//PHY constants   250kbps
#define aSymbolTime				0.000016
#define aMaxPHYPacketSize		127
#define aTurnaroundTime			12


/* PHY enumeration */
#define PHY_BUSY 				0x00
#define PHY_BUSY_RX			0x01
#define PHY_BUSY_TX			0x02
#define PHY_FORCE_TRX_OFF	0x03
#define PHY_IDLE				0x04
#define PHY_INVALID_PARAMETER		0x05
#define PHY_RX_ON				0x06
#define PHY_SUCCESS			0x07
#define PHY_TRX_OFF			0x08
#define PHY_TX_ON				0x09
#define PHY_UNSUPPORTED_ATTRIBUTE	0x0a

#define PHY_CCA_MODE1			0x01
#define PHY_CCA_MODE2			0x02
#define PHY_CCA_MODE3			0x03

/* PHY PIB attribute */
class MAC802_15_4_PHY_PIB
{
public:
	u_int32_t phyCurrentChannel;
	u_int32_t phyChannelSupported;
	u_char phyTransmitPower;
	u_int32_t phyCCAMode;
};


#define PHY_SHR_LEN			5	
#define PHY_PHR_LEN			1	
#define PHY_HDR_LEN			6

#define MAC_CMD_HDR_LEN			9	
#define MAC_DATA_HDR_LEN		11
#define MAC_ACK_HDR_LEN			5	
#define MAC_BEACON_HDR_LEN		11

// MAC Sublayer Constants 
#define aBaseSlotDuration				60
//#define aExtendedAddress		
#define aMaxBE								5
#define aMaxBeaconOverhead				75
#define aMaxBeaconPayloadLength		aMaxPHYPacketSize - aMaxBeaconOverhead
#define aGTSDescPersistenceTime		4
#define aMaxFrameOverhead				25
#define aMaxFrameResponseTime			1220
#define aMaxFrameRetries				3
#define aMaxLostBeacons					4
#define aMaxMACFrameSize				aMaxPHYPacketSize - aMaxFrameOverhead
#define aMaxSIFSFrameSize				18
#define aMinCAPLength					440
#define aMinLIFSPeriod					40
#define aMinSIFSPeriod					12
#define aNumSuperframeSlots			16
#define aResponseWaitTime				32 * aBaseSuperframeDuration
#define aUnitBackoffPeriod				20
#define aBaseSuperframeDuration		aBaseSlotDuration * aNumSuperframeSlots


// MAC Enumeration description
#define MAC_SUCCESS			0x00
#define MAC_BEACON_LOSS			0xe0
#define MAC_CHANNEL_ACCESS_FAILURE	0xe1
#define MAC_DENIED			0xe2
#define MAC_DISABLE_TRX_FAILURE		0xe3
#define MAC_FAILED_SECURITY_CHECK	0xe4
#define MAC_FRAME_TOO_LONG		0xe5
#define MAC_INVALID_GTS			0xe6
#define MAC_INVALID_HANDLE		0xe7
#define MAC_INVALID_PARAMETER		0xe8
#define MAC_NO_ACK			0xe9
#define MAC_NO_BEACON			0xea
#define MAC_NO_DATA			0xeb
#define MAC_NO_SHORT_ADDRESS		0xec
#define MAC_OUT_OF_CAP			0xed
#define MAC_PAN_ID_CONFLICT		0xee
#define MAC_REALIGNMENT			0xef
#define MAC_TRANSACTION_EXPIRED		0xf0
#define MAC_TRANSCATION_OVERFLOW	0xf1
#define MAC_TX_ACTIVE			0xf2
#define MAC_UNAVAILABLE_KEY		0xf3
#define MAC_UNSUPPORTED_ATTRIBUTE	0xf4

class ACLEntry
{
public:
//		  ACLExtendedAddress;
	u_int32_t ACLShortAddress;
	u_int32_t ACLPANId;
	u_int32_t ACKSecurityMaterialLength;
//		  ACLSecurityMaterial;
	u_int32_t ACLSecuritySuite;
};				

class MAC802_15_4_MAC_PIB
{
public:
	u_int32_t macAckWaitDuration;
	bool      macAssociationPermit;
	bool      macAutoRequest;
	bool      macBattLifeExt;
	u_int32_t macBattLifeExtPeriods;
	u_int32_t macBeaconPayload;          //
	u_int32_t macBeaconPayloadLength;
	u_int32_t macBeaconOrder;	
	u_int32_t macBeaconTxTime;
	u_int32_t macBSN;
	u_int32_t macCoordExtendedAddress;     //
	u_int32_t macCoordShortAddress;
	u_int32_t macDSN;
	u_int32_t macGTSPermit;
	u_int32_t macMaxCSMABackoffs;
	u_int32_t macMinBE;
	u_int32_t macPANId;
	bool      macPromiscuousMode;
	bool      macRxOnWhenIdle;
	u_int32_t macShortAddress;
	u_int32_t macSuperframeOrder;
	u_int32_t macTransactionPersistenceTime;
	ACLEntry  macACLEntryDescriptorSet;
	u_int32_t macACLEntryDescriptorSetSize;
	bool      macDefaultSecurity;
	u_int32_t macDefaultSecurityMaterialLength;
	u_int32_t macDefaultSecurityMaterial;     //
	u_int32_t macDefaultSecuritySuite;
	u_int32_t macSecurityMode;
	bool	  macPanCoordinator;
};

#define MAC_802_15_4_Type_Beacon 0x00
#define MAC_802_15_4_Type_Data   0x01
#define MAC_802_15_4_Type_Ack    0x02
#define MAC_802_15_4_Type_Cmd    0x03

struct MAC_802_15_4_frame_control 
{
	u_char frame_type;
	u_char security_enabled;
	u_char frame_pending;
	u_char ack_request;
	u_char intra_pan;
//	u_char reserved;
	u_char dst_addr_mode;
//	u_char reserved;
	u_char src_addr_mode;
};

#define PAN_identifier_none	0x00
#define PAN_identifier_rvsd	0x01
#define PAN_identifier_short	0x02
#define PAN_identifier_extended	0x03

struct hdr_mac802_15_4 
{
//	u_char		valid;
	struct MAC_802_15_4_frame_control	fc;
	u_int8_t	seq;
	u_int16_t	dst_PAN_identifier;
	u_char		dst[8];
	u_int16_t	src_PAN_identifier;
	u_char		src[8];

//	u_char		payload[100];
};
		
#define MFR_LEN			2

struct superframeField
{
	u_char		beacon_order;
	u_char		superframe_order;
	u_char		final_cap_slot;
	u_char		battery_life_extension;
	u_char		reserved;
	u_char		pan_coordinator;
	u_char		association_permit;
};

struct GTSSpec
{
	u_char		GTS_desc_count;
	u_char		reserved;
	u_char		GTS_permit;
};

struct GTSDirection
{
	u_char		GTS_direction_mask[7];
	u_char		reserved;
};

struct GTSList
{
	u_char		device_short_address;
	u_char		GTS_starting_slot;
	u_char		GTS_length;	
};

struct GTSField
{
	struct GTSSpec	GTS_spec;
	struct GTSDirection	GTS_directions;
	struct GTSList	GTS_list[7];
};	

struct pendingSpec
{
	u_char		num_short_pending;
//	u_char		reserved;
	u_char		num_extension_pending;
//	u_char		reserved;
};

struct PendingField
{
	struct pendingSpec	pending_addr_spec;
	u_char			addr_list[7]; 		//maximum pending address is 7
};

struct MAC_802_15_4_beacon_frame 
{
	struct MAC_802_15_4_frame_control	fc;
	u_int8_t	seq;
	u_int16_t	dst_PAN_identifier;
	u_char		dst[8];
	u_int16_t	src_PAN_identifier;
	u_char		src[8];
	
	struct superframeField	sf;
	struct GTSField		gts;
	struct PendingField	pending;
	u_char			payload[0];
	u_char			fcs[MFR_LEN];
};
	
/*
struct MAC_802_15_4_data_frame
{
	struct MAC_802_15_4_frame_control	fc;
	u_int8_t	seq;
	u_int16_t	dst_PAN_identifier;
	u_char		dst[8];
	u_int16_t	src_PAN_identifier;
	u_char		src[8];

};
*/

struct MAC_802_15_4_ack_frame
{
	struct MAC_802_15_4_frame_control	fc;
	u_int8_t	seq;
	u_int16_t	dst_PAN_identifier;
	u_char		dst[8];
	u_int16_t	src_PAN_identifier;
	u_char		src[8];
	u_int16_t	fcs[MFR_LEN];
};
	
#define Association_Request			0x01
#define Association_Response			0x02
#define Disassociation_Notification	0x03
#define Data_Request						0x04
#define PAN_conflict_Notification	0x05
#define Orphan_notification			0x06
#define Beacon_Request					0x07
#define Coordinator_realignment		0x08
#define GTS_Request						0x09

struct MAC_802_15_4_cmd_frame
{
	struct MAC_802_15_4_frame_control	fc;
	u_int8_t	seq;
	u_int16_t	dst_PAN_identifier;
	u_char		dst[8];
	u_int16_t	src_PAN_identifier;
	u_char		src[8];
	
	u_char		cmd_frame_identifier;
	u_char		payload[0];
	u_char		fcs[MFR_LEN];	
};

struct association_capability
{
	u_char		alternate_PAN_coordinator;
	u_char		device_type;
	u_char		power_source;
	u_char		receiver_idle;
	u_char		reserved;
	u_char		security_capability;
	u_char		allocate_address;
};

#define	Association_success	0x00
#define	PAN_at_capacity		0x01
#define PAN_access_denied	0x02

#define Coordinator_device_leave	0x01
#define Device_leave			0x02

struct GTSCharacter
{
	u_char		GTS_length;
	u_char		GTS_direction;
	u_char		GTS_type;
	u_char		reserved;
};

#endif
