/*
Gang Lu
Department of Electrical Engineering
University of Southern California
*/

/*The struture of the program is similar to  mac-802_11 from Monarch project in NS-2  
*/

#include "delay.h"
#include "connector.h"
#include "packet.h"
#include "random.h"
#include "mobilenode.h"
#include "wireless-phy.h"

#include "arp.h"
#include "ll.h"
#include "mac.h"
#include "cmu-trace.h"
#include "trace.h"

#include <mac-802_15_4-type.h>
#include <mac-802_15_4.h>
#include <mac-802_15_4-timers.h>

#define MIN(a,b)	a<b?a:b
//#define	TRUE		1
//#define	FALSE		0
#define HDR_MAC802_15_4(p) ((hdr_mac802_15_4 *)hdr_mac::access(p))

static MAC802_15_4_PHY_PIB PPIB = 
{
	11, 0x07FF, 0x01, PHY_CCA_MODE2
};

static  MAC802_15_4_MAC_PIB MPIB =
{
	54,	
	FALSE,
	TRUE,
	FALSE,
	6,					//macBattLifeExtPeriods
	0,
	0,
	4,					//macBeaconOrder
	0x000000,
	0,
	0x0, 
	0xffff,
	0,
	TRUE,				//macGTSPermit
	4,
	3,
	0xffff,
	FALSE,
	FALSE,
	0xffff,
	4,
	0x0f4,
	0,
	0x00,
	FALSE,
	0x15,
	0,
	0x00,
	0x00,
	FALSE
};
	
		
double SlotAlignment(double time, double slottime)
{
	double rmd= remainder(time, slottime);
	
	if(rmd>0.0)
		return slottime-rmd;
	else
//		return slottime+rmd;
		return 0-rmd;
}

void SuperframeTimer::expire(Event *e)
{
	mac_->superframeHandler();
} 

void GTSTimer::expire(Event *e)
{
	mac_->gtsHandler();
} 

double SuperframeTimer::rtime()
{
	rtime_=rtime_-(NOW-stime);

	return rtime_;
}

/* ======================================================================
   TCL Hooks for the simulator
   ====================================================================== */
static class Mac802_15_4Class : public TclClass {
public:
        Mac802_15_4Class() : TclClass("Mac/802_15_4"){}
        TclObject* create(int, const char*const*) {
                return (new Mac802_15_4(&PPIB, &MPIB));
        }
} class_mac802_15_4;


Mac802_15_4::Mac802_15_4(MAC802_15_4_PHY_PIB *p, MAC802_15_4_MAC_PIB* m) : Mac(), mhIF_(this), mhRecv_(this), mhSend_(this), mhDefer_(this, 0), beaconTimer_(this), mhSuperframe_(this), gtsTimer_(this)
{
	phypib_ = p;
	macpib_ = m;

	tx_state_ = rx_state_ = MAC802_15_IDLE;
	radio_state_ = RADIO_IDLE;
	trx_state_ = RX_ON;

	pktTx_ = 0;
	pktACK_ = 0;
	pktRx_ = 0;
	pktBn_ = 0;
	pktCmd_ = 0;
	dataRate_ = 250000;
	
//	macDSN = macBSN = 0;

	tx_resume_ = 0;

//	bind("macBeaconOrder", &(macpib_->macBeaconOrder));
//	bind("macSuperframenOrder", &(macpib_->macSuperframeOrder));

	macCAP = 15;
  	mhSuperframe_.CAPTime() = macCAP*aSymbolTime*aBaseSlotDuration*pow(2, macpib_->macSuperframeOrder);
        mhSuperframe_.CFPTime()=(15-macCAP)*aSymbolTime*aBaseSlotDuration*pow(2, macpib_->macSuperframeOrder);
	mhSuperframe_.SleepTime()=aSymbolTime*aBaseSuperframeDuration*(pow(2, macpib_->macBeaconOrder)-pow(2, macpib_->macSuperframeOrder));

	TxOptions = false;
	tracking = false;
	BI=aSymbolTime*aBaseSuperframeDuration*pow(2, macpib_->macBeaconOrder);

	coordinator = false;
	data_request = false;

	GTSAvailable=false;
}

int
Mac802_15_4::command(int argc, const char*const* argv)
{
      if (argc == 2) {
			if (strcmp(argv[1], "start")==0) {
				start();
         	return TCL_OK;
			}
			if (strcmp(argv[1], "GTS")==0) {
				TxOptions=true;
				tracking = true;
         	return TCL_OK;
			}
			if (strcmp(argv[1], "GTSAvailable")==0) {
				GTSAvailable = true;
            return TCL_OK;
			}
      }else if (argc == 3) {
                if (strcmp(argv[1], "trace-target") == 0) {
                        tracetarget_ = (NsObject*) TclObject::lookup(argv[2]);
                        if(tracetarget_ == 0)
                                return TCL_ERROR;
                        return TCL_OK;
                }
                if (strcmp(argv[1], "log-target") == 0) {
                        logtarget_ = (NsObject*) TclObject::lookup(argv[2]);
                        if(logtarget_ == 0)
                                return TCL_ERROR;
                        return TCL_OK;
                }
                if (strcmp(argv[1], "macBeaconOrder") == 0) {
			macpib_->macBeaconOrder = atoi(argv[2]);
			return TCL_OK;
		}
                if (strcmp(argv[1], "macSuperframeOrder") == 0) {
			macpib_->macSuperframeOrder = atoi(argv[2]);
			return TCL_OK;
		}
         if (strcmp(argv[1], "tracking") == 0) {
				tracking = atoi(argv[2]);
				return TCL_OK;
		}
                if (strcmp(argv[1], "macCAP") == 0) {
			macCAP = atoi(argv[2]);
			return TCL_OK;
		}
        }
        return Mac::command(argc, argv);
}
	

inline int
Mac802_15_4::CCA_request()
{
	if(trx_state_ == RX_ON)
	{
		if( radio_state_ == RADIO_IDLE )
			return PHY_IDLE;			

	 	if( radio_state_ == RADIO_RX )
			return PHY_BUSY;
	
		assert(0);
	}

	if(trx_state_ == TRX_OFF)
		return PHY_TRX_OFF;
	
	if(trx_state_ == TX_ON)	
		return PHY_TX_ON;	

	assert(0);
}

void Mac802_15_4::capture(Packet *p)
{
	Packet::free(p);
}

void Mac802_15_4::collision(Packet *p)
{
	switch(rx_state_)
	{
	case MAC802_15_RECV:
		rx_state_ = MAC802_15_COLL;
	case MAC802_15_COLL:
		assert(pktRx_);
		assert(mhRecv_.busy());
	
		if(txtime(p) > mhRecv_.expire())
		{
			mhRecv_.stop();
			discard(pktRx_, DROP_MAC_COLLISION);
			pktRx_ = p;
			mhRecv_.start(txtime(pktRx_));
		}
		else
			discard(p, DROP_MAC_COLLISION);
		break;
	default:
		assert(0);
	}
}

void Mac802_15_4::beaconHandler()
{
	mhSuperframe_.stime=NOW;
	sendBEACON();	
	beaconTimer_.resched(BI);
}

void Mac802_15_4::deferHandler()
{
//	assert(pktTx_ || pktACK_ || pktCmd_);
		
	if(pktACK_)
	{
		tx_state_ = MAC802_15_ACK;
		Transmit(pktACK_, txtime(pktACK_));
		return;
	}

	if(pktCmd_)
        {
                if(tx_resume_)
                {
                                CW=2;
                                tx_resume_ = 0;
                                double rTime = (Random::random() % ((int)pow(2, BE)-1))*aUnitBackoffPeriod*aSymbolTime;
                                if(rTime < mhSuperframe_.rtime())
                                        mhDefer_.start(SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod)+rTime);
                                return;

                }
                if(CCA_request()==PHY_IDLE)
                {
			assert(mhSuperframe_.status()==MAC_PAN_CAP);
                        if(CW==2)
                        {
                                if( (2*aSymbolTime*aUnitBackoffPeriod+txtime(pktCmd_)+macpib_->macAckWaitDuration*aSymbolTime) > mhSuperframe_.rtime() )
                                        return;
                        }
                        if(CW==0)
                        {
                                tx_state_ = MAC802_15_CMD;
                                Transmit(pktCmd_, txtime(pktCmd_)+macpib_->macAckWaitDuration*aSymbolTime);
                                return;
                        }
                        CW--;
                        mhDefer_.start(SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod));
                }
                else
                {
                        CW=2;
                        NB++;
                        BE=MIN(BE+1, aMaxBE);
                        tx_state_ = MAC802_15_IDLE;
                        if(NB>(int)macpib_->macMaxCSMABackoffs)
                                //error
                                assert(0);
                        else
                        {
                                double rTime = (Random::random() % ((int)pow(2, BE)-1))*aUnitBackoffPeriod*aSymbolTime;
                                if(SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod)+rTime < mhSuperframe_.rtime())
                                        mhDefer_.start(SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod)+rTime);
				else
					sleep();
                        }
                }
                return;
        }

	if(pktTx_)
	{
		if(tx_resume_ && !TxOptions)
		{
				CW=2;	
				tx_resume_ = 0;
				if(macpib_->macBattLifeExt)
					BE=MIN(2, macpib_->macMinBE);
				else
					BE=macpib_->macMinBE;
				double rTime = (Random::random() % ((int)pow(2, BE)-1))*aUnitBackoffPeriod*aSymbolTime;
				if(rTime + SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod) < mhSuperframe_.rtime())
					mhDefer_.start(SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod)+rTime);
				else
					sleep();
				return;
			
		}
		if(CCA_request()==PHY_IDLE)
		{
//			if(mhSuperframe_.status()!=MAC_PAN_CAP)
//				assert(mhSuperframe_.status()==MAC_PAN_CAP);
			if(CW==2)	
			{
				if( (2*aSymbolTime*aUnitBackoffPeriod+txtime(pktTx_)+macpib_->macAckWaitDuration*aSymbolTime) > mhSuperframe_.rtime() )
				{
					sleep();
					return;
				}
			}
			if(CW==0)
			{
				tx_state_ = MAC802_15_SEND;
				Transmit(pktTx_, txtime(pktTx_)+macpib_->macAckWaitDuration*aSymbolTime);
				return;
			}
			CW--;
			mhDefer_.start(SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod));
		}
		else
		{
			CW=2;
			NB++;
			BE=MIN(BE+1, aMaxBE);
			tx_state_ = MAC802_15_IDLE;
			if(NB>(int)macpib_->macMaxCSMABackoffs)
			{
				//error
//				assert(0);
				CW=2;
				NB=0;

				Packet::free(pktTx_);
				pktTx_=0;
				
				tx_resume();
			}
			else
			{
				double rTime = (Random::random() % ((int)pow(2, BE)-1))*aUnitBackoffPeriod*aSymbolTime;
				if(SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod)+rTime < mhSuperframe_.rtime())
					mhDefer_.start(SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod)+rTime);
				else
					sleep();
			}
		}
		return;
	}
				
}

void Mac802_15_4::tx_resume()
{
	if(!pktTx_ && callback_)
	{
		Handler *h = callback_;
		callback_ = 0;
		h->handle((Event *)0);
	}

	tx_state_ = MAC802_15_IDLE;
	
	return;
//	assert(0);
}

void Mac802_15_4::Transmit(Packet *p, double timeout)
{
	radio_state_ = RADIO_TX;
	trx_state_ = TX_ON;
	
/*
	if(rx_state_ != MAC802_15_IDLE) 
	{
		assert(tx_state_=MAC);
	}
*/

	downtarget_->recv(p->copy(), this);
	mhSend_.start(timeout);
	mhIF_.start(txtime(p)+aTurnaroundTime*aSymbolTime);
}

void Mac802_15_4::recvHandler()
{
	recv_timer();
}
		
void Mac802_15_4::sendHandler()
{
	send_timer();
}

void Mac802_15_4::txHandler()
{
	radio_state_ = RADIO_IDLE;
	trx_state_ = RX_ON;
//	tx_state_ = MAC802_15_IDLE;
}

void Mac802_15_4::recv(Packet *p, Handler *h)
{
	struct hdr_cmn *hdr = HDR_CMN(p);

	if(hdr->direction() == hdr_cmn::DOWN )
	{
		send(p, h);
		return;
	}

	if(radio_state_==RADIO_TX && hdr->error() == 0)
	{
		hdr->error() = 1;
	}

	if( rx_state_ == MAC802_15_IDLE)
	{
		if(radio_state_==RADIO_IDLE)
			radio_state_ = RADIO_RX;
		rx_state_ = MAC802_15_RECV;

//		if(mhDefer_.busy())
//			mhDefer_.stop();
		pktRx_ = p;
		mhRecv_.start(txtime(p));
	} 
	else
	{
		if(pktRx_->txinfo_.RxPr / p->txinfo_.RxPr >= p->txinfo_.CPThresh)
			capture(p);
		else
			collision(p);
	}
}


void Mac802_15_4::send(Packet *p, Handler *h)
{
	double rTime;
//	struct hdr_mac802_15_4* dh = HDR_MAC802_15_4(p);

	callback_ = h;
	createDATA(p);
	
	if(coordinator && !data_request)
		return;

	if( !TxOptions && mhSuperframe_.status()==MAC_PAN_CAP)
	{
		NB=0;
		CW=2;

		if(trx_state_==TRX_OFF)
			wakeup();

		if(macpib_->macBattLifeExt)
			BE=MIN(2, macpib_->macMinBE);
		else
			BE=macpib_->macMinBE;

		if( mhDefer_.busy()==0 )	
		{
			rTime = (Random::random() % ((int)pow(2, BE)-1))*aUnitBackoffPeriod*aSymbolTime;

			if(rTime+SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod) < mhSuperframe_.rtime() )
				mhDefer_.start(SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod)+rTime);
			else
				sleep();
		}
	}
	else if(mhSuperframe_.status()==MAC_PAN_LOST_BEACON)
	{
		if(trx_state_!=TRX_OFF)
			assert(trx_state_==TRX_OFF);
		assert(!tracking);

		if(trx_state_==TRX_OFF)
			wakeup();
	}

}

void Mac802_15_4::start()
{
	//set up superframe configuration, and start beacon tx.
	assert(pktBn_==0);
	
	coordinator=true;
//	double temp=pow(2, macpib_->macBeaconOrder);
	BI=aSymbolTime*aBaseSuperframeDuration*pow(2, macpib_->macBeaconOrder);
	
	mhSuperframe_.stime=NOW;
	sendBEACON();
	beaconTimer_.sched(BI);	
}

void Mac802_15_4::sendBEACON()
{
	createBEACON();

	assert(mhDefer_.busy()==0);
	if(radio_state_!=RADIO_IDLE)
	{
		fprintf(stderr, "index_=%d\n", index_);
		assert(radio_state_==RADIO_IDLE);	
	}
	if(tx_state_!=MAC802_15_IDLE)
		assert(tx_state_==MAC802_15_IDLE);
	assert(rx_state_==MAC802_15_IDLE);

	tx_state_=MAC802_15_BEACON;
	Transmit(pktBn_, txtime(pktBn_));	
}

void Mac802_15_4::createBEACON()
{
	Packet *p = Packet::alloc();
	hdr_cmn *mh = HDR_CMN(p);
	struct MAC_802_15_4_beacon_frame *dh = (struct MAC_802_15_4_beacon_frame*)p->access(hdr_mac::offset_);

	mh->size()=MAC_BEACON_HDR_LEN+PHY_HDR_LEN;
	mh->ptype()=PT_MAC;

	dh->fc.frame_type=MAC_802_15_4_Type_Beacon;
	dh->fc.security_enabled=FALSE;
	dh->fc.frame_pending = 0;
	dh->fc.ack_request = 0;
	dh->fc.intra_pan = 1;
	dh->fc.dst_addr_mode = PAN_identifier_short;
	dh->fc.src_addr_mode = PAN_identifier_short;

	dh->seq = macpib_->macBSN++;
	u_int32_t dst=MAC_BROADCAST;
	STORE4BYTE(&dst, (dh->dst));

	dh->sf.beacon_order = macpib_->macBeaconOrder;
	dh->sf.superframe_order = macpib_->macSuperframeOrder;
	dh->sf.final_cap_slot = macCAP;	
	dh->sf.battery_life_extension=macpib_->macBattLifeExt;
	dh->sf.pan_coordinator = macpib_->macPanCoordinator;
	dh->sf.association_permit = macpib_->macAssociationPermit;
	mh->size()+=2;

	dh->pending.pending_addr_spec.num_short_pending = 0;	
//	mh->size()+=;				//each one take 
	dh->pending.addr_list[0]=0;
//	dh->pending.addr_list[1]=2;


// You need to manually modify the following code to allocate GTS for different nodes in the PAN */
	mh->size()+=1;
	if(GTSAvailable)
	{
		dh->gts.GTS_spec.GTS_desc_count=1;
		if(dh->gts.GTS_spec.GTS_desc_count>0)
			mh->size()+=1+dh->gts.GTS_spec.GTS_desc_count*3;	//each GTS list takes 3 bytes
	}

	dh->gts.GTS_spec.GTS_permit = true;	
	dh->gts.GTS_directions.GTS_direction_mask[0]=0;
	dh->gts.GTS_list[0].device_short_address=12;
	dh->gts.GTS_list[0].GTS_starting_slot=macCAP;
	dh->gts.GTS_list[0].GTS_length=15-macCAP;
	
	dh->gts.GTS_directions.GTS_direction_mask[1]=0;
	dh->gts.GTS_list[1].device_short_address=2;
	dh->gts.GTS_list[1].GTS_starting_slot=13;
	dh->gts.GTS_list[1].GTS_length=2;

	pktBn_ = p;
	mh->txtime() = txtime(p);
}

void Mac802_15_4::createCMD(int cmd)
{
	Packet *p = Packet::alloc();
	hdr_cmn *mh = HDR_CMN(p);
	struct MAC_802_15_4_cmd_frame *dh = (struct MAC_802_15_4_cmd_frame*)p->access(hdr_mac::offset_);

	mh->size() = MAC_CMD_HDR_LEN+PHY_HDR_LEN;
	mh->ptype() = PT_MAC;

	u_int32_t dst=1;
	STORE4BYTE(&dst, (dh->dst));
	STORE4BYTE(&index_, (dh->src));

        dh->fc.frame_type=MAC_802_15_4_Type_Cmd;
        dh->fc.security_enabled=FALSE;
        dh->fc.frame_pending = 0;
        dh->fc.ack_request = 1;
        dh->fc.intra_pan = 1;
        dh->fc.dst_addr_mode = PAN_identifier_short;
        dh->fc.src_addr_mode = PAN_identifier_short;

        dh->seq = macpib_->macDSN++;

	switch(cmd)
	{
	case Data_Request:
		mh->size() += 1;
        	mh->txtime() = txtime(p);
		dh->cmd_frame_identifier = cmd;
		break;
	default:
		assert(0);
	}

	pktCmd_ = p;
}
		
void Mac802_15_4::recvBEACON(Packet *p)
{
	struct MAC_802_15_4_beacon_frame *dh = (struct MAC_802_15_4_beacon_frame*)p->access(hdr_mac::offset_);
	
	macpib_->macBeaconOrder = dh->sf.beacon_order;
	macpib_->macSuperframeOrder = dh->sf.superframe_order;
	macpib_->macBattLifeExt = dh->sf.battery_life_extension;
	macCAP = dh->sf.final_cap_slot;

	mhSuperframe_.CAPTime() = dh->sf.final_cap_slot*aSymbolTime*aBaseSlotDuration*pow(2, macpib_->macSuperframeOrder);
	mhSuperframe_.CFPTime()=(15-dh->sf.final_cap_slot)*aSymbolTime*aBaseSlotDuration*pow(2, macpib_->macSuperframeOrder);
	mhSuperframe_.SleepTime()=aSymbolTime*aBaseSuperframeDuration*(pow(2, macpib_->macBeaconOrder)-pow(2, macpib_->macSuperframeOrder));


	mhSuperframe_.status()=MAC_PAN_CAP;
	mhSuperframe_.sched(SlotAlignment(NOW, aSymbolTime*aBaseSlotDuration*pow(2, macpib_->macSuperframeOrder))+mhSuperframe_.CAPTime());
//	mhSuperframe_.sched(mhSuperframe_.CAPTime()-NOW);
	mhSuperframe_.stime = NOW;
	mhSuperframe_.rtime_ = SlotAlignment(NOW, aSymbolTime*aBaseSlotDuration*pow(2, macpib_->macSuperframeOrder))+mhSuperframe_.CAPTime();


	if(dh->pending.pending_addr_spec.num_short_pending!=0)
	{
		int i;
		for(i=0; i<dh->pending.pending_addr_spec.num_short_pending; i++)
		{
			if(dh->pending.addr_list[i]==index_)
				break;
		}
		if(i<dh->pending.pending_addr_spec.num_short_pending)
			createCMD(Data_Request);	
	}

	int i;
	for(i=0; i<dh->gts.GTS_spec.GTS_desc_count; i++)
	{
		if(dh->gts.GTS_list[i].device_short_address==index_)
		{
			gtsTimer_.length()=dh->gts.GTS_list[i].GTS_length*aSymbolTime*aBaseSlotDuration*pow(2, macpib_->macSuperframeOrder);
			double temp=dh->gts.GTS_list[i].GTS_starting_slot*aSymbolTime*aBaseSlotDuration*pow(2, macpib_->macSuperframeOrder);
			gtsTimer_.sched(SlotAlignment(NOW, aSymbolTime*aBaseSlotDuration*pow(2, macpib_->macSuperframeOrder))+temp);
			sleep();	
		}
	}
	
	if( !TxOptions && (pktTx_ || pktCmd_) )
        {
                NB=0;
                CW=2;

                if(macpib_->macBattLifeExt)
                        BE=MIN(2, macpib_->macMinBE);
                else
                        BE=macpib_->macMinBE;

                if( mhDefer_.busy()==0 )
                {
                        double rTime = (Random::random() % ((int)pow(2, BE)-1))*aUnitBackoffPeriod*aSymbolTime;
                        mhDefer_.start(SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod)+rTime);
                }
        }
	else
	{
		if(trx_state_!=TRX_OFF)
			sleep();
	}

	mac_log(p);
}

void Mac802_15_4::createDATA(Packet *p)
{
	hdr_cmn *mh = HDR_CMN(p);
	struct hdr_mac802_15_4 *dh = HDR_MAC802_15_4(p);
	
	assert(pktTx_==0);
	
	mh->size() += MAC_DATA_HDR_LEN+PHY_HDR_LEN;

	dh->fc.frame_type=MAC_802_15_4_Type_Data;
	dh->fc.security_enabled=FALSE;
	dh->fc.frame_pending = 0;
	dh->fc.ack_request = 1;
	dh->fc.intra_pan = 1;
	dh->fc.dst_addr_mode = PAN_identifier_short;
	dh->fc.src_addr_mode = PAN_identifier_short;
	
	dh->seq = macpib_->macDSN++;
	mh->txtime() = txtime(p);

	pktTx_ = p;	
}

void Mac802_15_4::createACK(int seq, int pending)
{
	Packet *p = Packet::alloc();
	hdr_cmn* mh = HDR_CMN(p);
	struct MAC_802_15_4_ack_frame *ah = (struct MAC_802_15_4_ack_frame *)p->access(hdr_mac::offset_);
	
	if(pktACK_)
		assert(pktACK_==0);
	mh->uid() = 0;
	mh->ptype() = PT_MAC;
	mh->size() = MAC_ACK_HDR_LEN+PHY_HDR_LEN;
	mh->iface() = -2;
	mh->error()=0;
	
	bzero(ah, sizeof(struct MAC_802_15_4_ack_frame));
	u_int32_t dst=seq;
	STORE4BYTE(&dst, (ah->dst));
	STORE4BYTE(&index_, (ah->src));
	
	ah->fc.frame_type = MAC_802_15_4_Type_Ack;
	ah->fc.security_enabled = 0;
	ah->fc.frame_pending = pending;
	ah->fc.ack_request = 1;
	ah->fc.intra_pan = 1;
	ah->fc.dst_addr_mode = PAN_identifier_short;
	ah->fc.src_addr_mode = PAN_identifier_short;
	
	ah->seq = seq;
	mh->txtime() = txtime(p);
	
	pktACK_ = p;
}

void Mac802_15_4::recv_timer()
{

	hdr_cmn *mh = HDR_CMN(pktRx_);
	hdr_mac802_15_4 *dh = HDR_MAC802_15_4(pktRx_);

	assert(pktRx_);
	if( (rx_state_ != MAC802_15_RECV) && (rx_state_!=MAC802_15_COLL) )
		assert((rx_state_ == MAC802_15_RECV)||(rx_state_==MAC802_15_COLL));

	/* Packet not received, used to set rx_state_ to perform CCA */
	if( (radio_state_ != RADIO_RX ) || mh->error() )
	{
		Packet::free(pktRx_);
		pktRx_ = 0;
		rx_state_ = MAC802_15_IDLE;
		radio_state_ = RADIO_IDLE;
		return;
	}

	/* Collision */
	if(rx_state_ == MAC802_15_COLL)
	{
		discard(pktRx_, DROP_MAC_COLLISION);
		pktRx_ = 0;
		rx_state_ = MAC802_15_IDLE;
		radio_state_ = RADIO_IDLE;
		return;
	}


	/* Filtering */
	if( (ETHER_ADDR(dh->dst) != index_ ) && (ETHER_ADDR(dh->dst)!=MAC_BROADCAST) )
	{
//		discard(pktRx_, "---");	
		Packet::free(pktRx_);
		pktRx_ =0;
		rx_state_ = MAC802_15_IDLE;
		radio_state_ = RADIO_IDLE;
		return;
	}
	

	u_char type = dh->fc.frame_type;

	switch(type)
	{
	case MAC_802_15_4_Type_Beacon:
//	        trace("r "TIME_FORMAT" _%d_ recv BEACON", NOW, index_);
		recvBEACON(pktRx_);
		break;
	case MAC_802_15_4_Type_Data:
		recvDATA(pktRx_);
		break;
	case MAC_802_15_4_Type_Ack:
		recvACK(pktRx_);
		break;
	case MAC_802_15_4_Type_Cmd:
//	        trace("r "TIME_FORMAT" _%d_ recv CMD", NOW, index_);
		recvCMD(pktRx_);
		break;
	default:
		assert(0);
	}

	pktRx_ = 0;
	rx_state_ = MAC802_15_IDLE;
	radio_state_ = RADIO_IDLE;
}
		
void Mac802_15_4::recvCMD(Packet* p)
{
//	struct hdr_cmn *mh = HDR_CMN(p);
	struct MAC_802_15_4_cmd_frame *dh = (struct MAC_802_15_4_cmd_frame *)p->access(hdr_mac::offset_);

	assert(coordinator);
	createACK(ETHER_ADDR(dh->src), pktTx_!=0);
		
        trace("r "TIME_FORMAT" _%d_ recv CMD: Data Request ", NOW, index_);
	mhDefer_.start(aTurnaroundTime*aSymbolTime+SlotAlignment(NOW+aTurnaroundTime*aSymbolTime, aSymbolTime*aUnitBackoffPeriod));

	data_request=true;
	return;
}

/*
void Mac802_15_4::recvBEACON(Packet *p)
{
	struct hdr_mac802_15_4
	return;
}
*/

void Mac802_15_4::recvDATA(Packet* p)
{
	struct hdr_mac802_15_4 *dh = HDR_MAC802_15_4(p);
	struct hdr_cmn *mh = HDR_CMN(p);

	mh->size() -= (MAC_DATA_HDR_LEN+PHY_HDR_LEN); 
	mh->num_forwards() += 1;

//	createACK(dh->seq);
	createACK(ETHER_ADDR(dh->src), 0);

	mhDefer_.start(aTurnaroundTime*aSymbolTime+SlotAlignment(NOW+aTurnaroundTime*aSymbolTime, aSymbolTime*aUnitBackoffPeriod));

	uptarget_->recv(p, (Handler*)0);
}
		
void Mac802_15_4::recvACK(Packet *p)
{
//	struct hdr_mac802_15_4 *dh = HDR_MAC802_15_4(p);
	struct hdr_cmn *mh = HDR_CMN(p);
	
	if( (tx_state_!=MAC802_15_SEND) && (tx_state_!=MAC802_15_CMD) )
	{
		discard(p, DROP_MAC_INVALID_STATE);
		return;
	}

	u_int32_t size = mh->size();	//only MPDU size, not PPDU size


	if(pktTx_)
	{
		assert(pktTx_);
		Packet::free(pktTx_); 
		pktTx_ = 0;

		mhSend_.stop();
		mac_log(p);
		tx_state_ = MAC802_15_IDLE;

		if(!TxOptions)
		{
			NB = 0; 
			CW = 2;
		
			if(size<aMaxSIFSFrameSize)
				mhDefer_.start(aMinSIFSPeriod*aSymbolTime);
			else
				mhDefer_.start(aMinLIFSPeriod*aSymbolTime);

			tx_resume_ = 1;
			tx_resume();
			if(!pktTx_)
				sleep();
		}
		else
		{
			tx_resume();
			sleep();
		}

		return;
	}
	
	if(pktCmd_)
	{
		struct MAC_802_15_4_cmd_frame *dh=(struct MAC_802_15_4_cmd_frame*)pktCmd_->access(hdr_mac::offset_);
		switch(dh->cmd_frame_identifier)
		{
		case Data_Request:
			break;
		default:
			assert(0);
		}

		mac_log(p);
	}
}
	
void Mac802_15_4::trace(char *fmt,...)
{
        va_list ap;

        if(!tracetarget_)
                return;
        va_start(ap, fmt);
        vsprintf(((Trace *)tracetarget_)->pt_->buffer(), fmt, ap);
        ((Trace *)tracetarget_)->pt_->dump();
        va_end(ap);
}

inline int Mac802_15_4::hdr_dst(char *hdr, int dst)
{
	struct hdr_mac802_15_4 *dh = (struct hdr_mac802_15_4*)hdr;
	
	if(dst > -2)
		STORE4BYTE(&dst, (dh->dst));
	
	return ETHER_ADDR(dh->dst);
}

inline int
Mac802_15_4::hdr_src(char* hdr, int src )
{
        struct hdr_mac802_15_4 *dh = (struct hdr_mac802_15_4*) hdr;

        if(src > -2)
                STORE4BYTE(&src, (dh->src));

        return ETHER_ADDR(dh->src);
}

void
Mac802_15_4::discard(Packet *p, const char* why)
{
        hdr_mac802_15_4* dh = HDR_MAC802_15_4(p);
        hdr_cmn *mh = HDR_CMN(p);


        /* if the rcvd pkt contains errors, a real MAC layer couldn't
           necessarily read any data from it, so we just toss it now */
        if(mh->error() != 0) {
                Packet::free(p);
                //p = 0;
                return;
        }

        switch(dh->fc.frame_type) 
	{
        case MAC_802_15_4_Type_Beacon:
                drop(p, why);
                return;
	case MAC_802_15_4_Type_Data:
                drop(p);
                break;
	case MAC_802_15_4_Type_Ack:
		drop(p);
		break;
	case MAC_802_15_4_Type_Cmd:
		drop(p);
		break;
        default:
                fprintf(stderr, "invalid MAC type (%x)\n", dh->fc.frame_type);
//                trace_pkt(p);
		assert(0);
        }
        Packet::free(p);
}

void Mac802_15_4::send_timer()
{
	switch(tx_state_)
	{
	case MAC802_15_BEACON:
		assert(pktBn_);
		assert(mhSuperframe_.status()==MAC_PAN_BEACON);
		mhSuperframe_.status()=MAC_PAN_CAP;
		mhSuperframe_.CAPTime() = (macCAP+1)*aSymbolTime*aBaseSlotDuration*pow(2, macpib_->macSuperframeOrder);

		mhSuperframe_.sched(mhSuperframe_.CAPTime()+mhSuperframe_.stime-NOW);
		break;
	case MAC802_15_CMD:
		assert(pktCmd_);
		Packet::free(pktCmd_);
		pktCmd_=0;
		break;
	case MAC802_15_SEND:
		assert(pktTx_);
		Packet::free(pktTx_);
		pktTx_=0;
		break;
	case MAC802_15_ACK:
		assert(pktACK_);
		Packet::free(pktACK_);
		pktACK_=0;

		if(coordinator && data_request)  
		//coordinator receive a data request command
		{
			data_request=false;
			if( pktTx_ && (mhSuperframe_.status()==MAC_PAN_CAP))
        		{
		                NB=0;
               			CW=2;

		                if(macpib_->macBattLifeExt)
               		        	BE=MIN(2, macpib_->macMinBE);
		                else
               			        BE=macpib_->macMinBE;

		                if( mhDefer_.busy()==0 )
		                {
		                        double rTime = (Random::random() % ((int)pow(2, BE)-1))*aUnitBackoffPeriod*aSymbolTime;
		                        if(rTime+SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod)<mhSuperframe_.rtime())
		                                mhDefer_.start(SlotAlignment(NOW, aSymbolTime*aUnitBackoffPeriod)+rTime);
		                }
		        }
			if(pktTx_)
				return;
		}
		break;
	case MAC802_15_IDLE:
		break;
	default:
		assert(0);
	}

//	deferHandler();
	tx_resume();
}

void Mac802_15_4::gtsHandler()
{
	assert(mhSuperframe_.status()==MAC_PAN_CFP);
	
	if(TxOptions && pktTx_)	
	{
		if(trx_state_ == TRX_OFF)
			wakeup();

		tx_state_ = MAC802_15_SEND;
		Transmit(pktTx_, txtime(pktTx_)+macpib_->macAckWaitDuration*aSymbolTime);
	}
}

void Mac802_15_4::superframeHandler()
{
	if(mhSuperframe_.status()==MAC_PAN_CAP)
	{
		mhSuperframe_.status()=MAC_PAN_CFP;
        	mhSuperframe_.CFPTime()=(15-macCAP)*aSymbolTime*aBaseSlotDuration*pow(2, macpib_->macSuperframeOrder);
		mhSuperframe_.resched(mhSuperframe_.CFPTime());
		return;
	}

	if(mhSuperframe_.status()==MAC_PAN_CFP)
	{
		mhSuperframe_.status()=MAC_PAN_SLEEP;
		if(mhSuperframe_.SleepTime()==0)
		{
			;
//			mhSuperframe_.resched(mhSuperframe_.SleepTime());
//			trx_state_=TRX_OFF;
//			radio_state_=RADIO_TRX_OFF;
//			rx_state_ = tx_state_ =  MAC802_15_IDLE;
//			sleep();
		}
		else
		{
			mhSuperframe_.resched(mhSuperframe_.SleepTime()-0.00001);
			if(trx_state_!=TRX_OFF)
				sleep();
//			trx_state_=TRX_OFF;
			radio_state_ = RADIO_TRX_OFF;
			return;
		}
	}

	if(mhSuperframe_.status()==MAC_PAN_SLEEP)
	{
		if(tracking || pktTx_)
		{
			mhSuperframe_.status()=MAC_PAN_BEACON;
			radio_state_ = RADIO_IDLE;
			if(trx_state_==TRX_OFF)
				wakeup();
//			trx_state_=RX_ON;	
		}
		else
			mhSuperframe_.status()=MAC_PAN_LOST_BEACON;
	}
	
}

void Mac802_15_4::wakeup()
{
	assert(TRX_OFF);	
        EnergyModel *em = netif_->node()->energy_model();
        em->node_on()=1;
        ((WirelessPhy *)netif_)->node_on();

	tx_state_ = rx_state_ = MAC802_15_IDLE;
	radio_state_ = RADIO_IDLE;
	trx_state_ = RX_ON;

//        trace("w "TIME_FORMAT" _%d_ go to recv", NOW, index_);
}

void Mac802_15_4::sleep()
{
	
	if(trx_state_==TRX_OFF)
		return;

	if(mhRecv_.busy())
		mhRecv_.stop();

        EnergyModel *em = netif_->node()->energy_model();
        em->node_on()=0;
        ((WirelessPhy *)netif_)->node_off();

	trx_state_ = TRX_OFF;
	rx_state_ = tx_state_ =  MAC802_15_IDLE;
	radio_state_ = RADIO_TRX_OFF;
	
/*
        if(mhDefer_.busy())
                assert(mhDefer_.busy()==0);
        if(mhBackoff_.busy()!=0)
                mhBackoff_.stop();
*/

//        trace("w "TIME_FORMAT" _%d_ go to sleep", NOW, index_);
//        pktCTRL_=0;
}
