/* 
Gang Lu
Department of Electrical Engineering
University of Southern California
*/

#ifndef MAC_802_15_4_H
#define MAC_802_15_4_H

#include "mac-802_15_4-timers.h"
#include "marshall.h"
#include "god.h"

#define MAC_PAN_BEACON		0x00
#define MAC_PAN_CAP			0x01
#define MAC_PAN_CFP			0x02
#define MAC_PAN_SLEEP		0x03
#define MAC_PAN_LOST_BEACON		0x04

enum Mac15State
{
	MAC802_15_IDLE        = 0x0000,
   MAC802_15_POLLING     = 0x0001,
   MAC802_15_RECV        = 0x0010,
   MAC802_15_SEND        = 0x0100,
   MAC802_15_ACK         = 0x0800,
   MAC802_15_COLL        = 0x1000,
	MAC802_15_BEACON      = 0x2000,
   MAC802_15_CMD         = 0x4000
};

enum RadioState
{
	RADIO_IDLE	= 0x00,
	RADIO_TRX_OFF	= 0x01,
	RADIO_TX	= 0x02,
	RADIO_RX	= 0x03,
	RADIO_TA	= 0x04		// turn around from Tx to Rx, or from Rx to Tx
};

enum TRXState
{
	RX_ON		= 0x00,
	TX_ON		= 0x01,
	TRX_OFF	= 0x02
};

class Mac802_15_4;
class SuperframeTimer : public TimerHandler {
public:
        SuperframeTimer(Mac802_15_4 *a) : TimerHandler()
        {
                mac_=a;
                status_=MAC_PAN_BEACON;
        }
        void expire(Event *e);

        double CAPTime_;
        double CFPTime_;
	double SleepTime_;
	double rtime_;
        int status_;

        inline int& status() { return (status_); }
        inline double& CAPTime() { return (CAPTime_); }
        inline double& CFPTime() { return (CFPTime_); }
        inline double& SleepTime() { return (SleepTime_); }

        double rtime();
        double stime;
protected:
        Mac802_15_4 *mac_;
};

class GTSTimer : public TimerHandler {
public:
        GTSTimer(Mac802_15_4 *a) : TimerHandler()
        {
                mac_=a;
                status_=MAC_PAN_BEACON;
        }
        void expire(Event *e);

        int status_;
	double length_;

        inline int& status() { return (status_); }
        inline double& length() { return (length_); }

protected:
        Mac802_15_4 *mac_;
};

class Mac802_15_4 : public Mac 
{
	friend class DeferTimer;
   friend class BackoffTimer;
   friend class IFTimer;
   friend class RxTimer;
   friend class TxTimer;
	friend class BeaconTimer;
	friend class SuperframeTimer;

public:
	Mac802_15_4(MAC802_15_4_PHY_PIB *p, MAC802_15_4_MAC_PIB *m);
	void		recv(Packet *p, Handler *h);
		
protected:
	int		command(int argc, const char*const* argv);

	/*
		timers' handlers
	*/
	void		recv_timer(void);
	void		send_timer(void);

	double	inline	txtime(Packet *p) {
		return 8.*(HDR_CMN(p)->size()) / dataRate_;
	}
	
	void		deferHandler();
	void		recvHandler();
	void		sendHandler();
	void		txHandler();

private:

	void mac_log(Packet *p) {
		logtarget_->recv(p, (Handler *)0);
	}

	
	/*
		Packet Transmission/Recption Functions
	*/
	void		send(Packet *p, Handler *h);
	void		createDATA(Packet *p);
	void		createACK(int seq, int pending);

	void		recvACK(Packet *p);
	void		recvDATA(Packet *p);
	void		recvCMD(Packet *p);

	void		Transmit(Packet *p, double timeout);
	void		tx_resume();

	void		capture(Packet *p);
	void		collision(Packet *p);
	
	void 		discard(Packet *p, const char* why);

	int		CCA_request();
	void		trace(char *fmt,...);
	
	inline int	hdr_src(char* hdr, int src);
	inline int	hdr_dst(char* hdr, int dst);

protected:
	MAC802_15_4_PHY_PIB		*phypib_;
	MAC802_15_4_MAC_PIB		*macpib_;


private:
	double		dataRate_;
	
	/*
		Mac Timers
	*/
	IFTimer		mhIF_;
//	NavTimer	mhNav_;
	RxTimer		mhRecv_;
	TxTimer		mhSend_;

	DeferTimer	mhDefer_;	
//	BackoffTimer	mhBackoff_;
//	DeferTimer	mhBackoff_;	

	BeaconTimer	beaconTimer_;
	SuperframeTimer	mhSuperframe_;
	GTSTimer	gtsTimer_;	

	int 		NB;
	int 		CW;
	int 		BE;

//	int		macDSN;
//	int		macBSN; 

	u_int16_t 	macShortAddress;

	Mac15State	rx_state_;
	Mac15State	tx_state_;
	RadioState	radio_state_;
	TRXState		trx_state_;
	int			tx_resume_;

	Packet		*pktTx_;
	Packet		*pktACK_;
	Packet		*pktCmd_;
	Packet		*pktBn_;


	NsObject*	logtarget_;
	NsObject*	tracetarget_;

	bool		coordinator;

public:
	
	/* Parameters for Beacon Frame */
//	u_int32_t	PANId;       		macpib_->macPANId;
//	u_int32_t	LogicalChannel; 	phypib_->LogicalChannel;
//	u_int32_t	BeaconOrder;		macpib_->macBeaconOrder;
//	u_int32_t	SuperframeOrder;	macpib_->macSuperframeOrder;	
	bool		PANCoordinator;	
//	bool		BatteryLifeExtension;	macpib_->macBattLifeExt;
	bool		CoordRealignment;
//	bool		SecurityEnable;		macpib_->macDefaultSecurity;
	bool		BeaconEnabled;
	bool		BeaconTracked;

	double		BI;			//Beacon Interval;
	u_int32_t	macCAP;			//CAP 
	bool		TxOptions;		//CAP or GTS?
	bool		tracking;		//Tracking Beacons?
	
	void		beaconHandler();
	void		sendBEACON();
	void		createBEACON();
	void		recvBEACON(Packet *p);

	void		createCMD(int cmd);
	
	void		superframeHandler();
	void		gtsHandler();
	void		start();

	void		wakeup();
	void		sleep();

	bool		GTSAvailable;
			
	/* Parameters for CMD Frame */
	bool		data_request;	
};

#endif

