# ======================================================================
# Define options
# ======================================================================
set opt(chan)		Channel/WirelessChannel
set opt(prop)		Propagation/TwoRayGround
set opt(netif)		Phy/WirelessPhy
set opt(mac)		Mac/802_15_4
set opt(ifq)		Queue/DropTail/PriQueue
set opt(rp)       SimRoute            ;# routing protocol script
set opt(ll)			LL
set opt(ant)      Antenna/OmniAntenna
set opt(err)		UniformErrorProc

set opt(x)		2200	;# X dimension of the topography
set opt(y)		2200		;# Y dimension of the topography

set opt(cp)		"./cbr-49-test-3"
set opt(sc)		"./scen-49-test"

set opt(ifqlen)		400		;# max packet in ifq, avoid buffer overflow
set opt(nn)				49		;# number of nodes
set opt(seed)	      1.0	
set opt(stop)			2001.01		;# simulation time
set opt(tr)		   	cbrgrid.tr	;# trace file
set opt(lm)             "off"           ;# log movement
set opt(energymodel)    EnergyModel     ;
set opt(initialenergy)  1000.0              ;# Initial energy in Joules
#set opt(logenergy)      "on"           ;# log energy every 150 seconds

# Set the traffic parameters
set opt(PktSize) 100
set opt(rate) 100

# Set superframe structure here
set opt(macBeaconOrder) 0 
set opt(macSuperframeOrder)     0
set opt(tracking)       0
set opt(macCAP)		15

# ======================================================================
# needs to be fixed later
#LL set mindelay_		50us
#LL set delay_			25us
#LL set bandwidth_	        0		;# not used

Queue/DropTail/PriQueue set Prefer_Routing_Protocols    1

# unity gain, omni-directional antennas
# set up the antennas to be centered in the node and 1.5 meters above it
#Antenna/OmniAntenna set X_ 0
#Antenna/OmniAntenna set Y_ 0
#Antenna/OmniAntenna set Z_ 1.5
#Antenna/OmniAntenna set Gt_ 1.0
#Antenna/OmniAntenna set Gr_ 1.0

# Initialize the SharedMedia interface with parameters to make
# it work like the 914MHz Lucent WaveLAN DSSS radio interface
#Phy/WirelessPhy set CPThresh_ 10.0
#Phy/WirelessPhy set CSThresh_ 1.559e-11
#Phy/WirelessPhy set RXThresh_ 3.652e-10
#Phy/WirelessPhy set CSThresh_ 1.559e-11
Phy/WirelessPhy set CSThresh_ 3.981e-10
Phy/WirelessPhy set RXThresh_ 3.981e-10
#Phy/WirelessPhy set bandwidth_ 11000000
#Phy/WirelessPhy set Pt_ 0.2818
#Phy/WirelessPhy set Pt_ 0.2018
#Phy/WirelessPhy set freq_ 914e+6 
#Phy/WirelessPhy set L_ 1.0

#Mac/802_11 set dataRate_   1000000

# ======================================================================

proc usage { argv0 }  {
	puts "Usage: $argv0"
	puts "\tmandatory arguments:"
	puts "\t\t\[-x MAXX\] \[-y MAXY\]"
	puts "\toptional arguments:"
	puts "\t\t\[-cp conn pattern\] \[-sc scenario\] \[-nn nodes\]"
	puts "\t\t\[-seed seed\] \[-stop sec\] \[-tr tracefile\]\n"
}


proc getopt {argc argv} {
	global opt
	lappend optlist cp nn seed sc stop tr x y

	for {set i 0} {$i < $argc} {incr i} {
		set arg [lindex $argv $i]
		if {[string range $arg 0 0] != "-"} continue

		set name [string range $arg 1 end]
		set opt($name) [lindex $argv [expr $i+1]]
	}
}

proc log-movement {} {
    global logtimer ns_ ns

    set ns $ns_
    source ../../tcl/mobility/timer.tcl
    Class LogTimer -superclass Timer
    LogTimer instproc timeout {} {
        global opt node_;
        for {set i 0} {$i < $opt(nn)} {incr i} {
            $node_($i) log-movement
        }
        $self sched 0.1
    }

    set logtimer [new LogTimer]
    $logtimer sched 0.1
}

proc UniformErrorProc {} {
    global opt
       
    set errObj [new ErrorModel]
    $errObj unit packet
    #$errObj FECstrength $opt(FECstrength)
    #$errObj datapktsize 1000
    #$errObj cntrlpktsize 80
    $errObj set rate_ 0.0
    return $errObj
}

proc finish {} {
        global ns_ tracefd
	global nf
        $ns_ flush-trace
        #Close the trace file
        close $nf
	close $tracefd
}

# ======================================================================
# Main Program
# ======================================================================
getopt $argc $argv
source ../tcl/lib/ns-cmutrace.tcl

# do the get opt again incase the routing protocol file added some more
# options to look for
getopt $argc $argv

if { $opt(x) == 0 || $opt(y) == 0 } {
	usage $argv0
	exit 1
}

if {$opt(seed) > 0} {
	puts "Seeding Random number generator with $opt(seed)\n"
	ns-random $opt(seed)
}

#
# Initialize Global Variables
#
set ns_		[new Simulator]
set topo	[new Topography]

set tracefd	[open $opt(tr) w]

#set nf [open out.nam w]
#$ns_ namtrace-all-wireless $nf $opt(x) $opt(y)

$topo load_flatgrid $opt(x) $opt(y)

$ns_ trace-all $tracefd
#$ns_ use-newtrace
#
# Create God
#
create-god $opt(nn)
set god_ [God instance]

#
# log the mobile nodes movements if desired
#
if { $opt(lm) == "on" } {
    log-movement
}

#
#  Create the specified number of nodes $opt(nn) and "attach" them
#  the channel.
#  Each routing protocol script is expected to have defined a proc
#  create-mobile-node that builds a mobile node and inserts it into the
#  array global $node_($i)
#

	#global node setting

        $ns_ node-config -adhocRouting $opt(rp) \
			 -llType $opt(ll) \
			 -macType $opt(mac) \
			 -ifqType $opt(ifq) \
			 -ifqLen $opt(ifqlen) \
			 -antType $opt(ant) \
			 -propType $opt(prop) \
			 -phyType $opt(netif) \
			 -channelType $opt(chan) \
			 -topoInstance $topo \
			 -agentTrace ON \
			 -routeTrace OFF \
			 -macTrace OFF  \
			 -initialEnergy $opt(initialenergy) \
			 -energyModel $opt(energymodel) 

	for {set i 0} {$i < $opt(nn) } {incr i} {
		set node_($i) [$ns_ node]	
		$node_($i) random-motion 0		;# disable random motion
		$god_ new_node $node_($i)

		set ragent_($i) [$node_($i) get-ragent]
		set mac_($i)	[$node_($i) get-mac 0]
#		$node_($i) log-movement
#		$ns_ initial_node_pos $node_($i) 20
	}
#
# Source the Connection and Movement scripts
#
if { $opt(cp) == "" } {
	puts "*** NOTE: no connection pattern specified."
        set opt(cp) "none"
} else {
	puts "Loading connection pattern..."
	source $opt(cp)
}


#
# Tell all the nodes when the simulation ends
#
for {set i } {$i < $opt(nn) } {incr i} {
    $ns_ at $opt(stop).000001 "$node_($i) reset";
}
$ns_ at $opt(stop).1 "puts \"NS EXITING...\" ; $ns_ halt"
$ns_ at $opt(stop) "finish"


if { $opt(sc) == "" } {
	puts "*** NOTE: no scenario file specified."
        set opt(sc) "none"
} else {
	puts "Loading scenario file..."
	source $opt(sc)
	puts "Load complete..."
}

puts $tracefd "M 0.0 nn $opt(nn) x $opt(x) y $opt(y) rp $opt(rp)"
puts $tracefd "M 0.0 sc $opt(sc) cp $opt(cp) seed $opt(seed)"
puts $tracefd "M 0.0 prop $opt(prop) ant $opt(ant)"

puts "Starting Simulation..."
$ns_ run
