/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1990-1997 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char rcsid[] =
    "@(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/common/agent.cc,v 1.74 2002/06/14 23:15:02 yuri Exp $ (LBL)";
#endif

#include <assert.h>
#include <stdlib.h>


#include "config.h"
#include "simagent.h"
#include "trace.h"
#include "ip.h"
#include "flags.h"
#include "address.h"
#ifdef HAVE_STL
#include "nix/hdr_nv.h"
#include "nix/nixnode.h"
#endif //HAVE_STL

#include "random.h"

#define HDR_SIMAGENT(p)		((struct hdr_simagent*)hdr_simagent::access(p))

int hdr_simagent::offset_;
static class SimAgentHeaderClass : public PacketHeaderClass {
public:
	SimAgentHeaderClass() : PacketHeaderClass("PacketHeader/SimAgent",
							sizeof(hdr_simagent)){
		bind_offset(&hdr_simagent::offset_);
	}
}class_hdr_simagent;	

static class SimAgentClass : public TclClass {
public:
	SimAgentClass() : TclClass("Agent/SimAgent") {} 
	TclObject* create(int, const char*const*) {
		return (new SimAgent(PT_SIMAGENT));
	}
} class_simagent;

void AgentTimer::expire(Event *e)
{
	a_->AgentTimeOut();
}

SimAgent::SimAgent(packet_t pkttype) : Agent(pkttype), queue_(), agent_timer_(this)
{
 	seqno_=0;
	sink_=-1;
	blocked_=0;
	bind("aggregation_", &aggregation_);
	bind("interval_", &interval_);
	bind("random_", &random_);
}

void SimAgent::AgentTimeOut()
{
	Packet *pkt=queue_.deque();		
	Handler *h=0;
	if(pkt)
	{
		hdr_ip *ih=HDR_IP(pkt);
		hdr_simagent *sh=HDR_SIMAGENT(pkt);
		send(pkt, h);	
		trace("s "TIME_FORMAT" AGT _%d_ _%d_ [%d]", NOW, THIS_NODE, ih->daddr(), sh->seqno());
		agent_timer_.resched(interval_+Random::uniform(interval_*random_));
	}
	else
		blocked_=0;
}

int SimAgent::command(int argc, const char*const* argv)
{
	if(argc==2)
	{
		if(strcmp(argv[1], "start-timer")==0)
		{
			agent_timer_.sched(interval_);
			return TCL_OK;
		}
	}else if(argc==3)
	{
		if(strcmp(argv[1], "tracetarget")==0){
			tracetarget_=(NsObject *)TclObject::lookup(argv[2]);
			if(tracetarget_==0)
				return TCL_ERROR;
			return TCL_OK;
		}
		if(strcmp(argv[1], "sink")==0)
		{
			sink_=atoi(argv[2]);
			return TCL_OK;
		}
		if(strcmp(argv[1], "aggregation")==0)
		{
			aggregation_=atoi(argv[2]);
			return TCL_OK;
		}
	}
	return (Agent::command(argc, argv));
}

void SimAgent::recv(Packet* p, Handler*)
{
	/*
	 * didn't expect packet (or we're a null agent?)
	 */
	hdr_cmn *cmh=HDR_CMN(p);
	if(cmh->ptype()==PT_SIMAGENT)
	{
		hdr_ip *ih=HDR_IP(p);
		hdr_simagent *sh=HDR_SIMAGENT(p);
		trace("r "TIME_FORMAT" AGT _%d_ _%d_ [%d]", NOW, ih->saddr(), THIS_NODE, sh->seqno());
		Packet::free(p);
	}
	else
	{
		Packet *pkt=allocpkt();
		hdr_cmn *ch=HDR_CMN(pkt);
		hdr_ip *ih=HDR_IP(pkt);
		hdr_simagent *sh=HDR_SIMAGENT(pkt);
		Handler *h=0;
	
		ch->size()=40*aggregation_;
		ch->ptype()=PT_SIMAGENT;
		ih->saddr()=THIS_NODE;
		ih->daddr()=sink_;
		sh->seqno()=seqno_++;
		
		queue_.enque(pkt);
		if(!blocked_)
		{
			pkt=queue_.deque();	
			send(pkt, h);	
			trace("s "TIME_FORMAT" AGT _%d_ _%d_ [%d]", NOW, THIS_NODE, ih->daddr(), sh->seqno());
//			agent_timer_.sched(interval_+Random::uniform(interval_*random_));
			agent_timer_.sched(interval_);
			blocked_=1;
		}
		Packet::free(p);
	}
}

void SimAgent::trace(char *fmt,...)
{
	va_list ap;
	if(!tracetarget_)
		return;
	va_start(ap, fmt);
	vsprintf(((Trace *)tracetarget_)->pt_->buffer(), fmt, ap);
	((Trace *)tracetarget_)->pt_->dump();
	va_end(ap);
}
