/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1993-1997 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/common/agent.h,v 1.35 2002/12/22 16:43:24 sfloyd Exp $ (LBL)
 */

#ifndef ns_sim_agent_h
#define ns_sim_agent_h

#include "connector.h"
#include "packet.h"
#include "queue.h"
#include "timer-handler.h"
#include "ns-process.h"
#include "agent.h"
#define TIME_FORMAT "%.15g"
// TIME_FORMAT is in basetrace.h, but including that header leads to problems


/* 
 * Note that timers are now implemented using timer-handler.{cc,h}
 */

#define THIS_NODE here_.addr_

struct hdr_simagent {
	int seqno_;
	
	static int offset_;
	inline static int& offset() { return offset_; }
	inline static hdr_simagent* access(const Packet* p) {
		return (hdr_simagent*)p->access(offset_);
	}

	inline int& seqno() { return seqno_; }
};

class SimAgent;

class AgentTimer : public TimerHandler {
public:
	AgentTimer(SimAgent *a) : TimerHandler() { a_=a; }
	void expire(Event *e);
protected:
	SimAgent *a_;
};

class SimAgent : public Agent {
 public:
	SimAgent(packet_t pktType);
	void recv(Packet*, Handler*);
	void AgentTimeOut();
	inline double interval() { return interval_; }

 protected:
	int command(int argc, const char*const* argv);
 private:
	int seqno_;
	int sink_;
	int aggregation_;
	int random_;
	int blocked_;
	PacketQueue queue_;
	AgentTimer agent_timer_;
	double interval_;
	NsObject *tracetarget_;
	void trace(char *fmt,...);
};

#endif

