#include "simroute-s.h"
#include "stcp_header.h"
#include "packet.h"

#if 0
int hdr_stcp::offset_;
static class StcpHeaderClass : public PacketHeaderClass {
public:
	StcpHeaderClass() : PacketHeaderClass("PacketHeader/STCP", 
					sizeof(hdr_stcp)) {
		bind_offset(&hdr_stcp::offset_);
	}
}class_stcphdr;
#endif

static class SimSrcRouteClass : public TclClass {
public:
	SimSrcRouteClass() : TclClass("Agent/SimSrcRoute") {}
	TclObject* create(int, const char*const*) {
		return(new SimSrcRouteAgent());
	}
}class_SimSrcRouteClass;

SimSrcRouteAgent::SimSrcRouteAgent() : Agent(PT_STCP)
{
	next_hop_=-1;
	seq_num_=0;
	stcp_type_=0;
	deadline_=0;
	bind("priority_type_", &priority_type_);
	bind("traceflag_", &traceflag_);
}

void SimSrcRouteAgent::recv(Packet* packet, Handler*)
{
	hdr_ip* ih = HDR_IP(packet);
	hdr_cmn* ch=HDR_CMN(packet);

	if(ih->daddr() == THIS_NODE)
		RecvData(packet);
	else if(ih->saddr() == THIS_NODE)
		SendData(packet);
	else if(ch->next_hop()==THIS_NODE)
		ForwardData(packet);
	else
		assert(0);

	return;
}

void SimSrcRouteAgent::RecvData(Packet *p)
{
	hdr_stcp *sh=HDR_STCP(p);
	hdr_ip *ih=HDR_IP(p);
	hdr_cmn *ch=HDR_CMN(p);

	if(ch->ptype()!=PT_ACK)
		trace("r "TIME_FORMAT" _%d_ LOG  --- %d TcpLog %d", NOW, THIS_NODE, sh->source_id_, sh->seq_num_);
	ih->dport()=sh->port_;
	port_dmux_->recv(p, (Handler*)0);
	return;
}

void SimSrcRouteAgent::SendData(Packet *p)
{
	hdr_ip* ih=HDR_IP(p);
	hdr_cmn* ch=HDR_CMN(p);
	hdr_stcp* sh=HDR_STCP(p);
	int i;
	
	if(ch->ptype()==PT_ACK)
	{
		ch->next_hop()=prev_hop_[ih->daddr()];
		ch->addr_type()=NS_AF_ILINK;
		ch->direction()=hdr_cmn::DOWN;
		sh->source_id_=THIS_NODE;
		sh->stcp_type_=2;
		sh->priority_=100;
		sh->port_=ih->dport();
		ih->saddr()=THIS_NODE;
		ih->dport()=RT_PORT;
		Scheduler::instance().schedule(ll_, p, 0.001);
		return;
	}

	for(i=0; i<20; i++)
		sh->source_route[i]=source_route[i];
	ch->size()+=HDR_STCP_LEN;
//	ch->next_hop()=next_hop_;
	ch->next_hop()=source_route[0];
	ch->addr_type()=NS_AF_ILINK;
	ch->direction()=hdr_cmn::DOWN;
	sh->source_id_=THIS_NODE;
	sh->seq_num_=seq_num_++;
	sh->num_hops_=0;	
	sh->priority_=0;
	sh->port_=ih->dport();
	sh->stcp_type_=stcp_type_;
	ih->saddr()=THIS_NODE;
	ih->dport()=RT_PORT;
	if(stcp_type_==1)	//real time with deadline
		sh->deadline_=NOW+deadline_;	

	trace("r "TIME_FORMAT" _%d_ LOG  --- %d TcpLog %d", NOW, THIS_NODE, sh->source_id_, sh->seq_num_);
	Scheduler::instance().schedule(ll_, p, 0.001);
}

void SimSrcRouteAgent::ForwardData(Packet *p)
{
	hdr_cmn *ch=HDR_CMN(p);
	hdr_stcp *sh=HDR_STCP(p);
	hdr_ip *ih=HDR_IP(p);

	if(ch->ptype()==PT_ACK)
	{
		ch->next_hop()=prev_hop_[ih->daddr()];
		ch->addr_type()=NS_AF_ILINK;
		ch->direction()=hdr_cmn::DOWN;
		Scheduler::instance().schedule(ll_, p, 0.001);
		return;
	}

//	ch->next_hop()=next_hop_;
	ch->addr_type()=NS_AF_ILINK;
	ch->direction()=hdr_cmn::DOWN;
	sh->num_hops_++;
	ch->next_hop()=sh->source_route[sh->num_hops_];
	if(priority_type_==0)
		sh->priority_=(sh->num_hops_);
	else	
		sh->priority_=sqrt(sh->num_hops_);

	trace("r "TIME_FORMAT" _%d_ LOG  --- %d TcpLog %d", NOW, THIS_NODE, sh->source_id_, sh->seq_num_);
	Scheduler::instance().schedule(ll_, p, 0.001);
}
	
int SimSrcRouteAgent::command(int argc, const char*const* argv)
{
	if(argc==2) {
		if (strcasecmp(argv[1], "reset")==0) {
			return Agent::command(argc, argv);
		}
		if (strcasecmp(argv[1], "start")==0) {
			//Start();
			return TCL_OK;
		}
	}
	else if(argc==3) {
		if(strcasecmp(argv[1], "tracetarget")==0) {
			tracetarget_ = (Trace *)TclObject::lookup(argv[2]);
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "port-dmux") ==0 ) {
			port_dmux_ = (NsObject *)TclObject::lookup(argv[2]);
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "add-ll") == 0) {
			ll_=(NsObject *)TclObject::lookup(argv[2]);
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "next-hop") == 0) {
			next_hop_=atoi(argv[2]);
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "stcp-type") == 0) {
			stcp_type_=atoi(argv[2]);
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "priority-type") == 0) {
			priority_type_=atoi(argv[2]);
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "deadline") == 0) {
			deadline_=(double)atoi(argv[2]);
			return TCL_OK;
		}
	}
	else if(argc==4)
	{
		if(strcasecmp(argv[1], "prev-hop") == 0) {
			prev_hop_[atoi(argv[2])]=atoi(argv[3]);
			return TCL_OK;
		}
	}	
	else if(argc==17)
	{
		if (strcasecmp(argv[1], "source-route")==0) {
			int i;
			for(i=0; i<15; i++)
				source_route[i]=atoi(argv[i+2]);
			return TCL_OK;
		}
	}
	return Agent::command(argc, argv);
}

void SimSrcRouteAgent::trace(char *fmt,...)
{
	va_list ap;
	
	if(!tracetarget_ || !traceflag_)
		return;

	va_start(ap, fmt);
	vsprintf(tracetarget_->pt_->buffer(), fmt, ap);
	tracetarget_->pt_->dump();
	va_end(ap);
}
