#include "simroute.h"
#include "packet.h"

#if 0
int hdr_stcp::offset_;
static class StcpHeaderClass : public PacketHeaderClass {
public:
	StcpHeaderClass() : PacketHeaderClass("PacketHeader/STCP", 
					sizeof(hdr_stcp)) {
		bind_offset(&hdr_stcp::offset_);
	}
}class_stcphdr;
#endif

static class SimRouteClass : public TclClass {
public:
	SimRouteClass() : TclClass("Agent/SimRoute") {}
	TclObject* create(int, const char*const*) {
		return(new SimRouteAgent());
	}
}class_SimRouteClass;

SimRouteAgent::SimRouteAgent() : Agent(PT_SIMAGENT)
{
	next_hop_=-1;
	seq_num_=0;
	stcp_type_=0;
	deadline_=0;
	delay_=0.0;
	nifs_=0;
}

void SimRouteAgent::recv(Packet* packet, Handler*)
{
	hdr_ip* ih = HDR_IP(packet);
	hdr_cmn* ch=HDR_CMN(packet);
	int index=THIS_NODE;

	if(ih->daddr() == THIS_NODE)
		RecvData(packet);
	else if(ih->saddr() == THIS_NODE)
		SendData(packet);
	else if(ch->next_hop()==THIS_NODE)
		ForwardData(packet);
	else
		assert(0);

	return;
}

void SimRouteAgent::use_ll(int num)
{
	ll_=all_[num];
}

void SimRouteAgent::RecvData(Packet *p)
{
	hdr_ip *ih=HDR_IP(p);
	hdr_cmn *ch=HDR_CMN(p);

//	if(ch->ptype()!=PT_ACK)
//		trace("r "TIME_FORMAT" _%d_ LOG  --- %d TcpLog %d", NOW, THIS_NODE, sh->source_id_, sh->seq_num_);
	port_dmux_->recv(p, (Handler*)0);
	return;
}

void SimRouteAgent::SendData(Packet *p)
{
	hdr_ip* ih=HDR_IP(p);
	hdr_cmn* ch=HDR_CMN(p);
	
	if(ch->ptype()==PT_ACK)
	{
		ch->next_hop()=prev_hop_[ih->daddr()];
		ch->addr_type()=NS_AF_ILINK;
		ch->direction()=hdr_cmn::DOWN;
		ih->saddr()=THIS_NODE;
		ih->dport()=RT_PORT;
		Scheduler::instance().schedule(ll_, p, delay_);
		return;
	}

//	ch->size()+=HDR_STCP_LEN;
	ch->next_hop()=next_hop_;
	ch->addr_type()=NS_AF_ILINK;
	ch->direction()=hdr_cmn::DOWN;
	ih->saddr()=THIS_NODE;

//	trace("r "TIME_FORMAT" _%d_ LOG  --- %d TcpLog %d", NOW, THIS_NODE, sh->source_id_, sh->seq_num_);
	Scheduler::instance().schedule(ll_, p, delay_);
}

void SimRouteAgent::ForwardData(Packet *p)
{
	hdr_cmn *ch=HDR_CMN(p);
	hdr_ip *ih=HDR_IP(p);

	if(ch->ptype()==PT_ACK)
	{
		ch->next_hop()=prev_hop_[ih->daddr()];
		ch->addr_type()=NS_AF_ILINK;
		ch->direction()=hdr_cmn::DOWN;
		Scheduler::instance().schedule(ll_, p, delay_);
		return;
	}

	ch->next_hop()=next_hop_;
	ch->addr_type()=NS_AF_ILINK;
	ch->direction()=hdr_cmn::DOWN;

//	trace("r "TIME_FORMAT" _%d_ LOG  --- %d TcpLog %d", NOW, THIS_NODE, sh->source_id_, sh->seq_num_);
	Scheduler::instance().schedule(ll_, p, delay_);
}
	
int SimRouteAgent::command(int argc, const char*const* argv)
{
	int i;
	if(argc==2) {
		if (strcasecmp(argv[1], "reset")==0) {
			return Agent::command(argc, argv);
		}
		if (strcasecmp(argv[1], "start")==0) {
			//Start();
			return TCL_OK;
		}
	}
	else if(argc==3) {
		if(strcasecmp(argv[1], "tracetarget")==0) {
			tracetarget_ = (Trace *)TclObject::lookup(argv[2]);
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "port-dmux") ==0 ) {
			port_dmux_ = (NsObject *)TclObject::lookup(argv[2]);
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "add-ll") == 0) {
			all_[nifs_]=(NsObject *)TclObject::lookup(argv[2]);
			nifs_++;
//			ll_=(NsObject *)TclObject::lookup(argv[2]);
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "use-ll") == 0) {
			i=atoi(argv[2]);
			ll_=all_[i];
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "next-hop") == 0) {
			next_hop_=atoi(argv[2]);
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "stcp-type") == 0) {
			stcp_type_=atoi(argv[2]);
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "priority-type") == 0) {
			priority_type_=atoi(argv[2]);
			return TCL_OK;
		}
		if(strcasecmp(argv[1], "deadline") == 0) {
			deadline_=(double)atoi(argv[2]);
			return TCL_OK;
		}
	}
	else if(argc==4)
	{
		if(strcasecmp(argv[1], "prev-hop") == 0) {
			prev_hop_[atoi(argv[2])]=atoi(argv[3]);
			return TCL_OK;
		}
	}	

	return Agent::command(argc, argv);
}

void SimRouteAgent::trace(char *fmt,...)
{
	va_list ap;
	
	if(!tracetarget_ || !traceflag_)
		return;

	va_start(ap, fmt);
	vsprintf(tracetarget_->pt_->buffer(), fmt, ap);
	tracetarget_->pt_->dump();
	va_end(ap);
}
